/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import cn.hutool.core.bean.BeanUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.req.AgentDataVisualListReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.resp.AgentDataVisualList;
import com.zayac.admin.resp.Statics;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.service.AgentDataVisualListService;
import com.zayac.admin.service.TeamService;
import com.zayac.admin.service.TelegramMessageService;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.req.AgentStatsReq;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.*;
import lombok.RequiredArgsConstructor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.concurrent.CompletableFuture;

@Component
@RequiredArgsConstructor
public class DailyReport {
    private final UserService userService;
    private final TeamService teamService;
    private final RoleService roleService;
    private final UserRoleService userRoleService;
    private final AccountService accountService;
    private final TelegramMessageService telegramMessageService;
    private final AgentDataVisualListService agentDataVisualListService;
    private final AgentStatsService agentStatsService;

    private static final String MINISTER_ROLE_CODE = "minister";

    @Scheduled(cron = "0 40 11,14,17,21 * * ?")
    public void teamAccountDailyReport() {
        LocalDateTime nowDateTime = LocalDateTime.now();
        sendDailyReport(LocalDate.now(), nowDateTime, nowDateTime, false);
    }

    @Scheduled(cron = "0 15 0 * * ?")
    public void dailySummarize() {
        LocalDate yesterday = LocalDate.now().minusDays(1);
        sendDailyReport(yesterday, yesterday.atStartOfDay(), LocalDateTime.of(yesterday, LocalTime.MAX), true);

    }

    private void sendDailyReport(LocalDate reportDate, LocalDateTime startDateTime, LocalDateTime endDateTime, boolean saveStatics) {
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());

        userIds.forEach(userId -> {
            UserDO ministerUser = userService.getById(userId);
            List<AccountResp> accounts = getTeamAccounts(ministerUser.getId());
            List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, ministerUser.getDeptId());

            accounts.forEach(accountResp -> {
                Team team = getLatestTeamInfo(accountResp, startDateTime, endDateTime);
                sendMinisterMessage(ministerUser, team.getList(), endDateTime);
            });

            AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder().build();
            deptUsers.forEach(deptUser -> {
                String message = getDeptUserMessage(deptUser, agentDataVisualListReq, reportDate, saveStatics);
                if (deptUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                    telegramMessageService.sendMessage(deptUser.getBotToken(), deptUser.getCountGroupId(), message);
                }
            });
        });
    }

    private List<AccountResp> getTeamAccounts(Long userId) {
        return accountService.getAccountsByUserId(userId, DisEnableStatusEnum.ENABLE)
                .stream()
                .filter(AccountResp::getIsTeam)
                .toList();
    }

    private Team getLatestTeamInfo(AccountResp accountResp, LocalDateTime startDateTime, LocalDateTime endDateTime) {
        TeamInfoReq teamInfoReq = TeamInfoReq.builder().startDate(startDateTime).endDate(endDateTime).build();
        return teamService.getLatestTeamInfoAsync(accountResp, teamInfoReq).join();
    }

    private void sendMinisterMessage(UserDO ministerUser, List<TeamAccount> list, LocalDateTime endDateTime) {
        if (ministerUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
            int totalNewMember = list.stream().mapToInt(TeamAccount::getSubMemberNum).sum();
            int totalNewFirstDeposit = list.stream().mapToInt(TeamAccount::getFirstDepositNum).sum();
            String percent = getPercent(totalNewFirstDeposit, totalNewMember);
            String message = String
                    .format("截至%s 注册:*%d*,新增:*%d* 转化率: *%s*", endDateTime, totalNewMember, totalNewFirstDeposit, percent);
            telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getCountGroupId(), message);
        }
    }

    private String getDeptUserMessage(UserDO deptUser,
                                      AgentDataVisualListReq agentDataVisualListReq,
                                      LocalDate reportDate,
                                      boolean saveStatics) {
        StringBuilder message = new StringBuilder();
        List<AccountResp> currUserAccounts = getTeamAccounts(deptUser.getId());

        currUserAccounts.forEach(currAccount -> {
            CompletableFuture<AgentDataVisualList> future = agentDataVisualListService
                    .getAgentDataVisualList(currAccount, agentDataVisualListReq);
            AgentDataVisualList agentData = future.join();
            Statics statics = agentData.getCurData()
                    .stream()
                    .filter(data -> data.getStaticsDate().equals(reportDate))
                    .findFirst()
                    .orElseThrow();
            message.append(currAccount.getUsername())
                    .append("\n")
                    .append("注册: ")
                    .append(statics.getIsNew())
                    .append("\n")
                    .append("新增: ")
                    .append(statics.getFirstCount())
                    .append("\n")
                    .append("日活: ")
                    .append(statics.getCountBets())
                    .append("\n\n");
            if (saveStatics) {
                AgentStatsReq agentStatsReq = new AgentStatsReq();
                BeanUtil.copyProperties(statics, agentStatsReq);
                agentStatsService.add(agentStatsReq);
            }
        });

        return message.toString();
    }

    public static String getPercent(int x, int y) {
        double d1 = x * 1.0;
        double d2 = y * 1.0;
        NumberFormat percentInstance = NumberFormat.getPercentInstance();
        percentInstance.setMinimumFractionDigits(2);
        return percentInstance.format(d1 / d2);
    }
}
